@@ -1,5 +1,5 @@
 # ext/indexable.py
-# Copyright (C) 2005-2024 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2026 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -21,9 +21,6 @@ The :mod:`~sqlalchemy.ext.indexable` extension provides
 :class:`_types.Indexable` typed column. In simple cases, it can be
 treated as a :class:`_schema.Column` - mapped attribute.
 
-
-.. versionadded:: 1.1
-
 Synopsis
 ========
 
@@ -38,19 +35,19 @@ as a dedicated attribute which behaves like a standalone column::
 
     Base = declarative_base()
 
+
     class Person(Base):
-        __tablename__ = 'person'
+        __tablename__ = "person"
 
         id = Column(Integer, primary_key=True)
         data = Column(JSON)
 
-        name = index_property('data', 'name')
-
+        name = index_property("data", "name")
 
 Above, the ``name`` attribute now behaves like a mapped column.   We
 can compose a new ``Person`` and set the value of ``name``::
 
-    >>> person = Person(name='Alchemist')
+    >>> person = Person(name="Alchemist")
 
 The value is now accessible::
 
@@ -61,11 +58,11 @@ Behind the scenes, the JSON field was initialized to a new blank dictionary
 and the field was set::
 
     >>> person.data
-    {"name": "Alchemist'}
+    {'name': 'Alchemist'}
 
 The field is mutable in place::
 
-    >>> person.name = 'Renamed'
+    >>> person.name = "Renamed"
     >>> person.name
     'Renamed'
     >>> person.data
@@ -89,18 +86,17 @@ A missing key will produce ``AttributeError``::
 
     >>> person = Person()
     >>> person.name
-    ...
     AttributeError: 'name'
 
 Unless you set a default value::
 
     >>> class Person(Base):
-    >>>     __tablename__ = 'person'
-    >>>
-    >>>     id = Column(Integer, primary_key=True)
-    >>>     data = Column(JSON)
-    >>>
-    >>>     name = index_property('data', 'name', default=None)  # See default
+    ...     __tablename__ = "person"
+    ...
+    ...     id = Column(Integer, primary_key=True)
+    ...     data = Column(JSON)
+    ...
+    ...     name = index_property("data", "name", default=None)  # See default
 
     >>> person = Person()
     >>> print(person.name)
@@ -113,11 +109,11 @@ an indexed SQL criteria::
 
     >>> from sqlalchemy.orm import Session
     >>> session = Session()
-    >>> query = session.query(Person).filter(Person.name == 'Alchemist')
+    >>> query = session.query(Person).filter(Person.name == "Alchemist")
 
 The above query is equivalent to::
 
-    >>> query = session.query(Person).filter(Person.data['name'] == 'Alchemist')
+    >>> query = session.query(Person).filter(Person.data["name"] == "Alchemist")
 
 Multiple :class:`.index_property` objects can be chained to produce
 multiple levels of indexing::
@@ -128,22 +124,25 @@ multiple levels of indexing::
 
     Base = declarative_base()
 
+
     class Person(Base):
-        __tablename__ = 'person'
+        __tablename__ = "person"
 
         id = Column(Integer, primary_key=True)
         data = Column(JSON)
 
-        birthday = index_property('data', 'birthday')
-        year = index_property('birthday', 'year')
-        month = index_property('birthday', 'month')
-        day = index_property('birthday', 'day')
+        birthday = index_property("data", "birthday")
+        year = index_property("birthday", "year")
+        month = index_property("birthday", "month")
+        day = index_property("birthday", "day")
 
 Above, a query such as::
 
-    q = session.query(Person).filter(Person.year == '1980')
+    q = session.query(Person).filter(Person.year == "1980")
 
-On a PostgreSQL backend, the above query will render as::
+On a PostgreSQL backend, the above query will render as:
+
+.. sourcecode:: sql
 
     SELECT person.id, person.data
     FROM person
@@ -200,13 +199,14 @@ version of :class:`_postgresql.JSON`::
 
     Base = declarative_base()
 
+
     class Person(Base):
-        __tablename__ = 'person'
+        __tablename__ = "person"
 
         id = Column(Integer, primary_key=True)
         data = Column(JSON)
 
-        age = pg_json_property('data', 'age', Integer)
+        age = pg_json_property("data", "age", Integer)
 
 The ``age`` attribute at the instance level works as before; however
 when rendering SQL, PostgreSQL's ``->>`` operator will be used
@@ -214,47 +214,61 @@ for indexed access, instead of the usual index operator of ``->``::
 
     >>> query = session.query(Person).filter(Person.age < 20)
 
-The above query will render::
+The above query will render:
+
+.. sourcecode:: sql
 
     SELECT person.id, person.data
     FROM person
     WHERE CAST(person.data ->> %(data_1)s AS INTEGER) < %(param_1)s
 
 """  # noqa
-from __future__ import absolute_import
+
+from __future__ import annotations
+
+from typing import Any
+from typing import Callable
+from typing import cast
+from typing import Optional
+from typing import TYPE_CHECKING
+from typing import TypeVar
+from typing import Union
 
 from .. import inspect
-from .. import util
 from ..ext.hybrid import hybrid_property
 from ..orm.attributes import flag_modified
 
+if TYPE_CHECKING:
+    from ..sql import SQLColumnExpression
+    from ..sql._typing import _HasClauseElement
+
 
 __all__ = ["index_property"]
 
+_T = TypeVar("_T")
 
-class index_property(hybrid_property):  # noqa
+
+class index_property(hybrid_property[_T]):
     """A property generator. The generated property describes an object
     attribute that corresponds to an :class:`_types.Indexable`
     column.
 
-    .. versionadded:: 1.1
-
     .. seealso::
 
         :mod:`sqlalchemy.ext.indexable`
 
     """
 
-    _NO_DEFAULT_ARGUMENT = object()
+    _NO_DEFAULT_ARGUMENT = cast(_T, object())
 
     def __init__(
         self,
-        attr_name,
-        index,
-        default=_NO_DEFAULT_ARGUMENT,
-        datatype=None,
-        mutable=True,
-        onebased=True,
+        attr_name: str,
+        index: Union[int, str],
+        default: _T = _NO_DEFAULT_ARGUMENT,
+        datatype: Optional[Callable[[], Any]] = None,
+        mutable: bool = True,
+        onebased: bool = True,
     ):
         """Create a new :class:`.index_property`.
 
@@ -280,13 +294,9 @@ class index_property(hybrid_property):  # noqa
         """
 
         if mutable:
-            super(index_property, self).__init__(
-                self.fget, self.fset, self.fdel, self.expr
-            )
+            super().__init__(self.fget, self.fset, self.fdel, self.expr)
         else:
-            super(index_property, self).__init__(
-                self.fget, None, None, self.expr
-            )
+            super().__init__(self.fget, None, None, self.expr)
         self.attr_name = attr_name
         self.index = index
         self.default = default
@@ -297,20 +307,20 @@ class index_property(hybrid_property):  # noqa
             self.datatype = datatype
         else:
             if is_numeric:
-                self.datatype = lambda: [None for x in range(index + 1)]
+                self.datatype = lambda: [None for x in range(index + 1)]  # type: ignore[operator]  # noqa: E501
             else:
                 self.datatype = dict
         self.onebased = onebased
 
-    def _fget_default(self, err=None):
+    def _fget_default(self, err: Optional[BaseException] = None) -> _T:
         if self.default == self._NO_DEFAULT_ARGUMENT:
-            util.raise_(AttributeError(self.attr_name), replace_context=err)
+            raise AttributeError(self.attr_name) from err
         else:
             return self.default
 
-    def fget(self, instance):
+    def fget(self, __instance: Any) -> _T:
         attr_name = self.attr_name
-        column_value = getattr(instance, attr_name)
+        column_value = getattr(__instance, attr_name)
         if column_value is None:
             return self._fget_default()
         try:
@@ -318,9 +328,9 @@ class index_property(hybrid_property):  # noqa
         except (KeyError, IndexError) as err:
             return self._fget_default(err)
         else:
-            return value
+            return value  # type: ignore[no-any-return]
 
-    def fset(self, instance, value):
+    def fset(self, instance: Any, value: _T) -> None:
         attr_name = self.attr_name
         column_value = getattr(instance, attr_name, None)
         if column_value is None:
@@ -331,7 +341,7 @@ class index_property(hybrid_property):  # noqa
         if attr_name in inspect(instance).mapper.attrs:
             flag_modified(instance, attr_name)
 
-    def fdel(self, instance):
+    def fdel(self, instance: Any) -> None:
         attr_name = self.attr_name
         column_value = getattr(instance, attr_name)
         if column_value is None:
@@ -339,14 +349,16 @@ class index_property(hybrid_property):  # noqa
         try:
             del column_value[self.index]
         except KeyError as err:
-            util.raise_(AttributeError(self.attr_name), replace_context=err)
+            raise AttributeError(self.attr_name) from err
         else:
             setattr(instance, attr_name, column_value)
             flag_modified(instance, attr_name)
 
-    def expr(self, model):
+    def expr(
+        self, model: Any
+    ) -> Union[_HasClauseElement[_T], SQLColumnExpression[_T]]:
         column = getattr(model, self.attr_name)
         index = self.index
         if self.onebased:
-            index += 1
-        return column[index]
+            index += 1  # type: ignore[operator]
+        return column[index]  # type: ignore[no-any-return]
