@@ -1,9 +1,25 @@
 # testing/plugin/plugin_base.py
-# Copyright (C) 2005-2024 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2026 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: https://www.opensource.org/licenses/mit-license.php
+# mypy: ignore-errors
+
+
+from __future__ import annotations
+
+import abc
+from argparse import Namespace
+import configparser
+import logging
+import os
+from pathlib import Path
+import re
+import sys
+from typing import Any
+
+from sqlalchemy.testing import asyncio
 
 """Testing extensions.
 
@@ -14,34 +30,12 @@ is pytest.
 
 """
 
-from __future__ import absolute_import
-
-import abc
-import logging
-import re
-import sys
-
 # flag which indicates we are in the SQLAlchemy testing suite,
 # and not that of Alembic or a third party dialect.
 bootstrapped_as_sqlalchemy = False
 
 log = logging.getLogger("sqlalchemy.testing.plugin_base")
 
-
-py3k = sys.version_info >= (3, 0)
-
-if py3k:
-    import configparser
-
-    ABC = abc.ABC
-else:
-    import ConfigParser as configparser
-    import collections as collections_abc  # noqa
-
-    class ABC(object):
-        __metaclass__ = abc.ABCMeta
-
-
 # late imports
 fixtures = None
 engines = None
@@ -59,7 +53,7 @@ file_config = None
 logging = None
 include_tags = set()
 exclude_tags = set()
-options = None
+options: Namespace = None  # type: ignore
 
 
 def setup_options(make_option):
@@ -96,7 +90,7 @@ def setup_options(make_option):
         action="append",
         type=str,
         dest="dburi",
-        help="Database uri.  Multiple OK, " "first one is run by default.",
+        help="Database uri.  Multiple OK, first one is run by default.",
     )
     make_option(
         "--dbdriver",
@@ -116,25 +110,39 @@ def setup_options(make_option):
     make_option(
         "--disable-asyncio",
         action="store_true",
-        help="disable test / fixtures / provisoning running in asyncio",
+        help="disable test / fixtures / provisioning running in asyncio",
     )
     make_option(
         "--backend-only",
-        action="store_true",
-        dest="backend_only",
-        help="Run only tests marked with __backend__ or __sparse_backend__",
+        action="callback",
+        zeroarg_callback=_set_tag_include("backend"),
+        help=(
+            "Run only tests marked with __backend__ or __sparse_backend__; "
+            "this is now equivalent to the pytest -m backend mark expression"
+        ),
     )
     make_option(
         "--nomemory",
-        action="store_true",
-        dest="nomemory",
-        help="Don't run memory profiling tests",
+        action="callback",
+        zeroarg_callback=_set_tag_exclude("memory_intensive"),
+        help="Don't run memory profiling tests; "
+        "this is now equivalent to the pytest -m 'not memory_intensive' "
+        "mark expression",
     )
     make_option(
         "--notimingintensive",
-        action="store_true",
-        dest="notimingintensive",
-        help="Don't run timing intensive tests",
+        action="callback",
+        zeroarg_callback=_set_tag_exclude("timing_intensive"),
+        help="Don't run timing intensive tests; "
+        "this is now equivalent to the pytest -m 'not timing_intensive' "
+        "mark expression",
+    )
+    make_option(
+        "--nomypy",
+        action="callback",
+        zeroarg_callback=_set_tag_exclude("mypy"),
+        help="Don't run mypy typing tests; "
+        "this is now equivalent to the pytest -m 'not mypy' mark expression",
     )
     make_option(
         "--profile-sort",
@@ -149,12 +157,6 @@ def setup_options(make_option):
         dest="profiledump",
         help="Filename where a single profile run will be dumped",
     )
-    make_option(
-        "--postgresql-templatedb",
-        type=str,
-        help="name of template database to use for PostgreSQL "
-        "CREATE DATABASE (defaults to current database)",
-    )
     make_option(
         "--low-connections",
         action="store_true",
@@ -169,14 +171,6 @@ def setup_options(make_option):
         help="write out generated follower idents to <file>, "
         "when -n<num> is used",
     )
-    make_option(
-        "--reversetop",
-        action="store_true",
-        dest="reversetop",
-        default=False,
-        help="Use a random-ordering set implementation in the ORM "
-        "(helps reveal dependency issues)",
-    )
     make_option(
         "--requirements",
         action="callback",
@@ -184,27 +178,21 @@ def setup_options(make_option):
         callback=_requirements_opt,
         help="requirements class for testing, overrides setup.cfg",
     )
-    make_option(
-        "--with-cdecimal",
-        action="store_true",
-        dest="cdecimal",
-        default=False,
-        help="Monkeypatch the cdecimal library into Python 'decimal' "
-        "for all tests",
-    )
     make_option(
         "--include-tag",
         action="callback",
         callback=_include_tag,
         type=str,
-        help="Include tests with tag <tag>",
+        help="Include tests with tag <tag>; "
+        "legacy, use pytest -m 'tag' instead",
     )
     make_option(
         "--exclude-tag",
         action="callback",
         callback=_exclude_tag,
         type=str,
-        help="Exclude tests with tag <tag>",
+        help="Exclude tests with tag <tag>; "
+        "legacy, use pytest -m 'not tag' instead",
     )
     make_option(
         "--write-profiles",
@@ -235,6 +223,18 @@ def setup_options(make_option):
         help="Additional test directories to add to the mypy tests. "
         "This is used only when running mypy tests. Multiple OK",
     )
+    # db specific options
+    make_option(
+        "--postgresql-templatedb",
+        type=str,
+        help="name of template database to use for PostgreSQL "
+        "CREATE DATABASE (defaults to current database)",
+    )
+    make_option(
+        "--oracledb-thick-mode",
+        action="store_true",
+        help="enables the 'thick mode' when testing with oracle+oracledb",
+    )
 
 
 def configure_follower(follower_ident):
@@ -254,15 +254,9 @@ def memoize_important_follower_config(dict_):
 
     This invokes in the parent process after normal config is set up.
 
-    This is necessary as pytest seems to not be using forking, so we
-    start with nothing in memory, *but* it isn't running our argparse
-    callables, so we have to just copy all of that over.
+    Hook is currently not used.
 
     """
-    dict_["memoized_config"] = {
-        "include_tags": include_tags,
-        "exclude_tags": exclude_tags,
-    }
 
 
 def restore_important_follower_config(dict_):
@@ -270,16 +264,17 @@ def restore_important_follower_config(dict_):
 
     This invokes in the follower process.
 
+    Hook is currently not used.
+
     """
-    global include_tags, exclude_tags
-    include_tags.update(dict_["memoized_config"]["include_tags"])
-    exclude_tags.update(dict_["memoized_config"]["exclude_tags"])
 
 
-def read_config():
+def read_config(root_path):
     global file_config
     file_config = configparser.ConfigParser()
-    file_config.read(["setup.cfg", "test.cfg"])
+    file_config.read(
+        [str(root_path / "setup.cfg"), str(root_path / "test.cfg")]
+    )
 
 
 def pre_begin(opt):
@@ -326,6 +321,10 @@ def _log(opt_str, value, parser):
 
 
 def _list_dbs(*args):
+    if file_config is None:
+        # assume the current working directory is the one containing the
+        # setup file
+        read_config(Path.cwd())
     print("Available --db options (use --dburi to override)")
     for macro in sorted(file_config.options("db")):
         print("%20s\t%s" % (macro, file_config.get("db", macro)))
@@ -336,6 +335,20 @@ def _requirements_opt(opt_str, value, parser):
     _setup_requirements(value)
 
 
+def _set_tag_include(tag):
+    def _do_include_tag(opt_str, value, parser):
+        _include_tag(opt_str, tag, parser)
+
+    return _do_include_tag
+
+
+def _set_tag_exclude(tag):
+    def _do_exclude_tag(opt_str, value, parser):
+        _exclude_tag(opt_str, tag, parser)
+
+    return _do_exclude_tag
+
+
 def _exclude_tag(opt_str, value, parser):
     exclude_tags.add(value.replace("-", "_"))
 
@@ -365,23 +378,33 @@ def _setup_options(opt, file_config):
 
 
 @pre
-def _set_nomemory(opt, file_config):
-    if opt.nomemory:
-        exclude_tags.add("memory_intensive")
+def _register_sqlite_numeric_dialect(opt, file_config):
+    from sqlalchemy.dialects import registry
+
+    registry.register(
+        "sqlite.pysqlite_numeric",
+        "sqlalchemy.dialects.sqlite.pysqlite",
+        "_SQLiteDialect_pysqlite_numeric",
+    )
+    registry.register(
+        "sqlite.pysqlite_dollar",
+        "sqlalchemy.dialects.sqlite.pysqlite",
+        "_SQLiteDialect_pysqlite_dollar",
+    )
 
 
-@pre
-def _set_notimingintensive(opt, file_config):
-    if opt.notimingintensive:
-        exclude_tags.add("timing_intensive")
+@post
+def __ensure_cext(opt, file_config):
+    if os.environ.get("REQUIRE_SQLALCHEMY_CEXT", "0") == "1":
+        from sqlalchemy.util import has_compiled_ext
 
-
-@pre
-def _monkeypatch_cdecimal(options, file_config):
-    if options.cdecimal:
-        import cdecimal
-
-        sys.modules["decimal"] = cdecimal
+        try:
+            has_compiled_ext(raise_=True)
+        except ImportError as err:
+            raise AssertionError(
+                "REQUIRE_SQLALCHEMY_CEXT is set but can't import the "
+                "cython extensions"
+            ) from err
 
 
 @post
@@ -393,18 +416,16 @@ def _init_symbols(options, file_config):
 
 @pre
 def _set_disable_asyncio(opt, file_config):
-    if opt.disable_asyncio or not py3k:
-        from sqlalchemy.testing import asyncio
-
+    if opt.disable_asyncio:
         asyncio.ENABLE_ASYNCIO = False
 
 
 @post
 def _engine_uri(options, file_config):
-
     from sqlalchemy import testing
     from sqlalchemy.testing import config
     from sqlalchemy.testing import provision
+    from sqlalchemy.engine import url as sa_url
 
     if options.dburi:
         db_urls = list(options.dburi)
@@ -429,15 +450,19 @@ def _engine_uri(options, file_config):
 
     config._current = None
 
+    if options.write_idents and provision.FOLLOWER_IDENT:
+        for db_url in [sa_url.make_url(db_url) for db_url in db_urls]:
+            with open(options.write_idents, "a") as file_:
+                file_.write(
+                    f"{provision.FOLLOWER_IDENT} "
+                    f"{db_url.render_as_string(hide_password=False)}\n"
+                )
+
     expanded_urls = list(provision.generate_db_urls(db_urls, extra_drivers))
 
     for db_url in expanded_urls:
         log.info("Adding database URL: %s", db_url)
 
-        if options.write_idents and provision.FOLLOWER_IDENT:
-            with open(options.write_idents, "a") as file_:
-                file_.write(provision.FOLLOWER_IDENT + " " + db_url + "\n")
-
         cfg = provision.setup_config(
             db_url, options, file_config, provision.FOLLOWER_IDENT
         )
@@ -447,7 +472,6 @@ def _engine_uri(options, file_config):
 
 @post
 def _requirements(options, file_config):
-
     requirement_cls = file_config.get("sqla_testing", "requirement_cls")
     _setup_requirements(requirement_cls)
 
@@ -456,9 +480,6 @@ def _setup_requirements(argument):
     from sqlalchemy.testing import config
     from sqlalchemy import testing
 
-    if config.requirements is not None:
-        return
-
     modname, clsname = argument.split(":")
 
     # importlib.import_module() only introduced in 2.7, a little
@@ -484,14 +505,6 @@ def _prep_testing_database(options, file_config):
             provision.drop_all_schema_objects(cfg, cfg.db)
 
 
-@post
-def _reverse_topological(options, file_config):
-    if options.reversetop:
-        from sqlalchemy.orm.util import randomize_unitofwork
-
-        randomize_unitofwork()
-
-
 @post
 def _post_setup_options(opt, file_config):
     from sqlalchemy.testing import config
@@ -516,13 +529,6 @@ def want_class(name, cls):
         return False
     elif name.startswith("_"):
         return False
-    elif (
-        config.options.backend_only
-        and not getattr(cls, "__backend__", False)
-        and not getattr(cls, "__sparse_backend__", False)
-        and not getattr(cls, "__only_on__", False)
-    ):
-        return False
     else:
         return True
 
@@ -532,34 +538,21 @@ def want_method(cls, fn):
         return False
     elif fn.__module__ is None:
         return False
-    elif include_tags:
-        return (
-            hasattr(cls, "__tags__")
-            and exclusions.tags(cls.__tags__).include_test(
-                include_tags, exclude_tags
-            )
-        ) or (
-            hasattr(fn, "_sa_exclusion_extend")
-            and fn._sa_exclusion_extend.include_test(
-                include_tags, exclude_tags
-            )
-        )
-    elif exclude_tags and hasattr(cls, "__tags__"):
-        return exclusions.tags(cls.__tags__).include_test(
-            include_tags, exclude_tags
-        )
-    elif exclude_tags and hasattr(fn, "_sa_exclusion_extend"):
-        return fn._sa_exclusion_extend.include_test(include_tags, exclude_tags)
     else:
         return True
 
 
-def generate_sub_tests(cls, module):
-    if getattr(cls, "__backend__", False) or getattr(
-        cls, "__sparse_backend__", False
+def generate_sub_tests(cls, module, markers):
+    if (
+        "backend" in markers
+        or "sparse_backend" in markers
+        or "sparse_driver_backend" in markers
     ):
-        sparse = getattr(cls, "__sparse_backend__", False)
-        for cfg in _possible_configs_for_cls(cls, sparse=sparse):
+        sparse = "sparse_backend" in markers
+        sparse_driver = "sparse_driver_backend" in markers
+        for cfg in _possible_configs_for_cls(
+            cls, sparse=sparse, sparse_driver=sparse_driver
+        ):
             orig_name = cls.__name__
 
             # we can have special chars in these names except for the
@@ -595,8 +588,8 @@ def stop_test_class(cls):
 
 
 def stop_test_class_outside_fixtures(cls):
-    engines.testing_reaper.stop_test_class_outside_fixtures()
     provision.stop_test_class_outside_fixtures(config, config.db, cls)
+    engines.testing_reaper.stop_test_class_outside_fixtures()
     try:
         if not options.low_connections:
             assertions.global_cleanup_assertions()
@@ -624,7 +617,6 @@ def _setup_engine(cls):
 
 
 def before_test(test, test_module_name, test_class, test_name):
-
     # format looks like:
     # "test.aaa_profiling.test_compiler.CompileTest.test_update_whereclause"
 
@@ -644,7 +636,9 @@ def after_test_fixtures(test):
     engines.testing_reaper.after_test_outside_fixtures(test)
 
 
-def _possible_configs_for_cls(cls, reasons=None, sparse=False):
+def _possible_configs_for_cls(
+    cls, reasons=None, sparse=False, sparse_driver=False
+):
     all_configs = set(config.Config.all_configs())
 
     if cls.__unsupported_on__:
@@ -675,6 +669,12 @@ def _possible_configs_for_cls(cls, reasons=None, sparse=False):
                         reasons.extend(skip_reasons)
                     break
 
+                warnings = check.matching_warnings(config_obj)
+                if warnings:
+                    cls.__warnings__ = getattr(
+                        cls, "__warnings__", ()
+                    ) + tuple(warnings)
+
     if hasattr(cls, "__prefer_requires__"):
         non_preferred = set()
         requirements = config.requirements
@@ -692,20 +692,54 @@ def _possible_configs_for_cls(cls, reasons=None, sparse=False):
         # sorted so we get the same backend each time selecting the highest
         # server version info.
         per_dialect = {}
-        for cfg in reversed(
+
+        sorted_all_configs = reversed(
             sorted(
                 all_configs,
                 key=lambda cfg: (
+                    "z" if cfg.is_default_dialect else "a",
                     cfg.db.name,
                     cfg.db.driver,
                     cfg.db.dialect.server_version_info,
+                    cfg.db.dialect.is_async,
                 ),
             )
-        ):
+        )
+
+        for cfg in sorted_all_configs:
             db = cfg.db.name
             if db not in per_dialect:
                 per_dialect[db] = cfg
         return per_dialect.values()
+    elif sparse_driver:
+        # a more liberal form of "sparse" that will select for one driver,
+        # but still return for multiple database servers
+
+        dbs = {}
+
+        sorted_all_configs = list(
+            reversed(
+                sorted(
+                    all_configs,
+                    key=lambda cfg: (
+                        cfg.db.name,
+                        cfg.db.driver,
+                        cfg.db.dialect.server_version_info,
+                        cfg.db.dialect.is_async,
+                    ),
+                )
+            )
+        )
+
+        for cfg in sorted_all_configs:
+            key = (cfg.db.name, cfg.db.dialect.server_version_info)
+            if key in dbs and dbs[key].is_default_dialect:
+                continue
+            else:
+                dbs[key] = cfg
+
+        chosen_cfgs = set(dbs.values())
+        return [cfg for cfg in sorted_all_configs if cfg in chosen_cfgs]
 
     return all_configs
 
@@ -722,7 +756,8 @@ def _do_skips(cls):
                 )
 
     if not all_configs:
-        msg = "'%s' unsupported on any DB implementation %s%s" % (
+        msg = "'%s.%s' unsupported on any DB implementation %s%s" % (
+            cls.__module__,
             cls.__name__,
             ", ".join(
                 "'%s(%s)+%s'"
@@ -756,7 +791,7 @@ def _setup_config(config_obj, ctx):
     config._current.push(config_obj, testing)
 
 
-class FixtureFunctions(ABC):
+class FixtureFunctions(abc.ABC):
     @abc.abstractmethod
     def skip_test_exception(self, *arg, **kw):
         raise NotImplementedError()
@@ -777,7 +812,11 @@ class FixtureFunctions(ABC):
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def mark_base_test_class(self):
+    def mark_base_test_class(self) -> Any:
+        raise NotImplementedError()
+
+    @abc.abstractproperty
+    def add_to_marker(self):
         raise NotImplementedError()
 
 
